@@ -439,126 +439,122 @@ public class PdfCopy extends PdfWriter {
 			// changes to the PdfReader will be copied, below, into the PdfWriter;
 			//
 			{
-				PdfArray annots= (PdfArray)PdfReader.getPdfObject(thePage.get(PdfName.ANNOTS));
+				PdfObject annots= PdfReader.getPdfObject(thePage.get(PdfName.ANNOTS));
 				if( annots!= null && annots.isArray() ) {
-					ArrayList annots_arr= annots.getArrayList();
-					for( int ii= 0; ii< annots_arr.size(); ++ii ) {
+					ArrayList<PdfObject> annots_arr= ((PdfArray)annots).getArrayList();
+					for( PdfObject annot_ref_obj : annots_arr ) {
 						// an annotation may be direct or indirect; ours must be indirect
-						PdfObject annot_obj= (PdfObject)annots_arr.get(ii);
-						if( annot_obj!= null && annot_obj.isIndirect() ) {
-							PdfIndirectReference annot_ref= (PdfIndirectReference)annot_obj;
-							if( annot_ref!= null ) {
-								PdfDictionary annot= (PdfDictionary)PdfReader.getPdfObject(annot_ref);
-								if( annot!= null && annot.isDictionary() ) {
-									PdfName subtype= (PdfName)PdfReader.getPdfObject(annot.get(PdfName.SUBTYPE));
-									if( subtype!= null && subtype.isName() && subtype.equals(PdfName.WIDGET) ) {
-										// we have a form field
+						if( annot_ref_obj!= null && annot_ref_obj.isIndirect() ) {
+							PdfIndirectReference annot_ref= (PdfIndirectReference)annot_ref_obj;
+							PdfObject annot_obj= PdfReader.getPdfObject(annot_ref);
+							if( annot_obj!= null && annot_obj.isDictionary() ) {
+								PdfDictionary annot = (PdfDictionary)annot_obj;
+								PdfObject subtype= PdfReader.getPdfObject(annot.get(PdfName.SUBTYPE));
+								if( PdfName.WIDGET.equals(subtype) ) {
+									// we have a form field
 
-										// get its full name
-										//
-										String full_name= ""; // construct a full name from partial names using '.', e.g.: foo.bar.
-										String top_name= "";
-										boolean is_unicode_b= false; // if names are unicode, they must all be unicode
-										PdfString tt= (PdfString)PdfReader.getPdfObject(annot.get(PdfName.T));
-										if( tt!= null && tt.isString() ) {
+									// get its full name
+									//
+									String full_name= ""; // construct a full name from partial names using '.', e.g.: foo.bar.
+									String top_name= "";
+									boolean is_unicode_b= false; // if names are unicode, they must all be unicode
+									PdfObject tt_obj= PdfReader.getPdfObject(annot.get(PdfName.T));
+									if( tt_obj!= null && tt_obj.isString() ) {
+										PdfString tt = (PdfString)tt_obj;
+										top_name= tt.toString();
+										is_unicode_b= ( is_unicode_b || PdfString.isUnicode( tt.getBytes() ) );
+									}
+									//
+									// dig upwards, parent-wise; replace annot as we go with the
+									// top-most form field dictionary
+									PdfIndirectReference parent_ref=
+										(PdfIndirectReference)annot.get(PdfName.PARENT);
+									while( parent_ref!= null && parent_ref.isIndirect() ) {
+										annot_ref= parent_ref;
+										annot= (PdfDictionary)PdfReader.getPdfObject(annot_ref);
+										parent_ref= (PdfIndirectReference)annot.get(PdfName.PARENT);
+
+										tt_obj= PdfReader.getPdfObject(annot.get(PdfName.T));
+										if( tt_obj!= null && tt_obj.isString() ) {
+											PdfString tt = (PdfString)tt_obj;
+											if( top_name.length()!= 0 ) {
+												full_name+= top_name;
+												full_name+= ".";
+											}
 											top_name= tt.toString();
 											is_unicode_b= ( is_unicode_b || PdfString.isUnicode( tt.getBytes() ) );
 										}
-										//
-										// dig upwards, parent-wise; replace annot as we go with the
-										// top-most form field dictionary
-										PdfIndirectReference parent_ref= 
-											(PdfIndirectReference)annot.get(PdfName.PARENT);
-										while( parent_ref!= null && parent_ref.isIndirect() )
-											{
-												annot_ref= parent_ref;
-												annot= (PdfDictionary)PdfReader.getPdfObject(annot_ref);
-												parent_ref= (PdfIndirectReference)annot.get(PdfName.PARENT);
-
-												tt= (PdfString)PdfReader.getPdfObject(annot.get(PdfName.T));
-												if( tt!= null && tt.isString() ) {
-													if( top_name.length()!= 0 ) {
-														full_name+= top_name;
-														full_name+= ".";
-													}
-													top_name= tt.toString();
-												}
-
-												is_unicode_b= ( is_unicode_b || PdfString.isUnicode( tt.getBytes() ) );
-											}
+									}
 									
-										// once we have seen a top-level field parent, we wave
-										// it through and assume that it harbors no illegal field duplicates;
-										// this is good, because sometimes authors want a single field
-										// represented by more than one annotation on the page; this logic
-										// respects that programming
+									// once we have seen a top-level field parent, we wave
+									// it through and assume that it harbors no illegal field duplicates;
+									// this is good, because sometimes authors want a single field
+									// represented by more than one annotation on the page; this logic
+									// respects that programming
+									//
+									//System.err.println( full_name+ top_name+ "." ); // debug
+									if( readerData.allNames.contains( top_name ) ) {
+										// a parent we have seen or created in this reader
+										this.fullFormFieldNames.add( full_name+ top_name+ "." ); // tally
+									}
+									else if( this.fullFormFieldNames.contains( full_name+ top_name+ "." ) ) {
+										// insert new, top-most parent
+
+										// name for new parent
+										int new_parent_name_ii= 1;
+										String new_parent_name= Integer.toString( new_parent_name_ii );
+										while( this.fullFormFieldNames.contains( full_name+ top_name+ "."+ new_parent_name+ "." ) ||
+											   this.topFormFieldNames.contains( new_parent_name ) &&
+											   !readerData.newNamesKids.containsKey( new_parent_name ) )
+											{
+												new_parent_name= Integer.toString( ++new_parent_name_ii );
+											}
+
+										PdfIndirectReference new_parent_ref= null;
+										PdfArray new_parent_kids= null;
 										//
-										//System.err.println( full_name+ top_name+ "." ); // debug
-										if( readerData.allNames.contains( top_name ) )
-											{ // a parent we have seen or created in this reader
-												this.fullFormFieldNames.add( full_name+ top_name+ "." ); // tally
-											}
-										else {
-											if( this.fullFormFieldNames.contains( full_name+ top_name+ "." ) ) {
-												// insert new, top-most parent
-
-												// name for new parent
-												int new_parent_name_ii= 1;
-												String new_parent_name= Integer.toString( new_parent_name_ii );
-												while( this.fullFormFieldNames.contains( full_name+ top_name+ "."+ new_parent_name+ "." ) ||
-													   this.topFormFieldNames.contains( new_parent_name ) &&
-													   !readerData.newNamesKids.containsKey( new_parent_name ) )
-													{
-														new_parent_name= Integer.toString( ++new_parent_name_ii );
-													}
-
-												PdfIndirectReference new_parent_ref= null;
-												PdfArray new_parent_kids= null;
-												//
-												if( readerData.newNamesKids.containsKey( new_parent_name ) ) {
-													// a new parent we already created
-													new_parent_ref= (PdfIndirectReference)
-														readerData.newNamesRefs.get( new_parent_name );
-													new_parent_kids= (PdfArray)
-														readerData.newNamesKids.get( new_parent_name );
-												}
-												else { // create a new parent using this name
-													PdfDictionary new_parent= new PdfDictionary();
-													PdfString new_parent_name_pdf= new PdfString( new_parent_name );
-													if( is_unicode_b ) { // if names are unicode, they must all be unicode
-														new_parent_name_pdf= new PdfString( new_parent_name, PdfObject.TEXT_UNICODE );
-													}
-													new_parent_ref= reader.getPRIndirectReference( new_parent );
-													new_parent.put( PdfName.T, new_parent_name_pdf );
-											
-													new_parent_kids= new PdfArray();
-													PdfIndirectReference new_parent_kids_ref= 
-														reader.getPRIndirectReference( new_parent_kids );
-													new_parent.put(PdfName.KIDS, new_parent_kids_ref);
-
-													// tally new parent
-													readerData.newNamesRefs.put( new_parent_name, new_parent_ref );
-													readerData.newNamesKids.put( new_parent_name, new_parent_kids );
-													readerData.allNames.add( new_parent_name );
-													this.topFormFieldNames.add( new_parent_name );
-												}
-
-												// wire annot and new parent together
-												annot.put( PdfName.PARENT, new_parent_ref );
-												new_parent_kids.add( annot_ref ); // the new parent must point at the field, too
-
-												// tally full field name
-												this.fullFormFieldNames.add( full_name+ top_name+ "."+ new_parent_name+ "." );
-											}
-											else {
-												// tally parent
-												readerData.allNames.add( top_name );
-												this.topFormFieldNames.add( top_name );
-
-												// tally full field name
-												this.fullFormFieldNames.add( full_name+ top_name+ "." );
-											}
+										if( readerData.newNamesKids.containsKey( new_parent_name ) ) {
+											// a new parent we already created
+											new_parent_ref= (PdfIndirectReference)
+												readerData.newNamesRefs.get( new_parent_name );
+											new_parent_kids= (PdfArray)
+												readerData.newNamesKids.get( new_parent_name );
 										}
+										else { // create a new parent using this name
+											PdfDictionary new_parent= new PdfDictionary();
+											PdfString new_parent_name_pdf= new PdfString( new_parent_name );
+											if( is_unicode_b ) { // if names are unicode, they must all be unicode
+												new_parent_name_pdf= new PdfString( new_parent_name, PdfObject.TEXT_UNICODE );
+											}
+											new_parent_ref= reader.getPRIndirectReference( new_parent );
+											new_parent.put( PdfName.T, new_parent_name_pdf );
+
+											new_parent_kids= new PdfArray();
+											PdfIndirectReference new_parent_kids_ref=
+												reader.getPRIndirectReference( new_parent_kids );
+											new_parent.put(PdfName.KIDS, new_parent_kids_ref);
+
+											// tally new parent
+											readerData.newNamesRefs.put( new_parent_name, new_parent_ref );
+											readerData.newNamesKids.put( new_parent_name, new_parent_kids );
+											readerData.allNames.add( new_parent_name );
+											this.topFormFieldNames.add( new_parent_name );
+										}
+
+										// wire annot and new parent together
+										annot.put( PdfName.PARENT, new_parent_ref );
+										new_parent_kids.add( annot_ref ); // the new parent must point at the field, too
+
+										// tally full field name
+										this.fullFormFieldNames.add( full_name+ top_name+ "."+ new_parent_name+ "." );
+									}
+									else {
+										// tally parent
+										readerData.allNames.add( top_name );
+										this.topFormFieldNames.add( top_name );
+
+										// tally full field name
+										this.fullFormFieldNames.add( full_name+ top_name+ "." );
 									}
 								}
 							}
@@ -579,35 +575,33 @@ public class PdfCopy extends PdfWriter {
 			//
 			// dig down to annot, and then dig up to topmost Parent
 			{
-				PdfArray annots= (PdfArray)PdfReader.getPdfObject(thePage.get(PdfName.ANNOTS));
+				PdfObject annots= PdfReader.getPdfObject(thePage.get(PdfName.ANNOTS));
 				if( annots!= null && annots.isArray() ) {
-					ArrayList annots_arr= annots.getArrayList();
-					for( int ii= 0; ii< annots_arr.size(); ++ii ) {
+					ArrayList<PdfObject> annots_arr= ((PdfArray)annots).getArrayList();
+					for( PdfObject annot_ref_obj : annots_arr ) {
 						// an annotation may be direct or indirect; ours must be indirect
-						PdfObject annot_obj= (PdfObject)annots_arr.get(ii);
-						if( annot_obj!= null && annot_obj.isIndirect() ) {
-							PdfIndirectReference annot_ref= (PdfIndirectReference)annots_arr.get(ii);
-							if( annot_ref!= null ) {
-								PdfDictionary annot= (PdfDictionary)PdfReader.getPdfObject(annot_ref);
-								if( annot!= null && annot.isDictionary() ) {
-									PdfName subtype= (PdfName)PdfReader.getPdfObject(annot.get(PdfName.SUBTYPE));
-									if( subtype!= null && subtype.isName() && subtype.equals(PdfName.WIDGET) ) {
-										// we have a form field
+						if( annot_ref_obj!= null && annot_ref_obj.isIndirect() ) {
+							PdfIndirectReference annot_ref= (PdfIndirectReference)annot_ref_obj;
+							PdfObject annot_obj= PdfReader.getPdfObject(annot_ref);
+							if( annot_obj!= null && annot_obj.isDictionary() ) {
+								PdfDictionary annot= (PdfDictionary)annot_obj;
+								PdfObject subtype= PdfReader.getPdfObject(annot.get(PdfName.SUBTYPE));
+								if( PdfName.WIDGET.equals(subtype) ) {
+									// we have a form field
 
-										// dig upwards, parent-wise
-										PdfIndirectReference parent_ref= 
-											(PdfIndirectReference)annot.get(PdfName.PARENT);
-										while( parent_ref!= null && parent_ref.isIndirect() ) {
-											annot_ref= parent_ref;
-											annot= (PdfDictionary)PdfReader.getPdfObject(annot_ref);
-											parent_ref= (PdfIndirectReference)annot.get(PdfName.PARENT);
-										}
-								
-										RefKey annot_key= new RefKey(annot_ref);
-										IndirectReferences annot_iRef= (IndirectReferences)indirects.get(annot_key);
-										PdfAcroForm acroForm= this.getAcroForm();
-										acroForm.addDocumentField( annot_iRef.getRef() );
+									// dig upwards, parent-wise
+									PdfIndirectReference parent_ref=
+										(PdfIndirectReference)annot.get(PdfName.PARENT);
+									while( parent_ref!= null && parent_ref.isIndirect() ) {
+										annot_ref= parent_ref;
+										annot= (PdfDictionary)PdfReader.getPdfObject(annot_ref);
+										parent_ref= (PdfIndirectReference)annot.get(PdfName.PARENT);
 									}
+								
+									RefKey annot_key= new RefKey(annot_ref);
+									IndirectReferences annot_iRef= (IndirectReferences)indirects.get(annot_key);
+									PdfAcroForm acroForm= this.getAcroForm();
+									acroForm.addDocumentField( annot_iRef.getRef() );
 								}
 							}
 						}
@@ -620,12 +614,15 @@ public class PdfCopy extends PdfWriter {
 			//
 			// pdftk-1.10: I noticed that the PdfAcroForm.isValid() apprears to stomp on this (TODO)
 			//
-			PdfDictionary catalog= reader.getCatalog();
-			if( catalog!= null && catalog.isDictionary() ) {
-				PdfDictionary acroForm= (PdfDictionary)PdfReader.getPdfObject(catalog.get(PdfName.ACROFORM));
-				if( acroForm!= null && acroForm.isDictionary() ) {
-					PdfDictionary dr= (PdfDictionary)PdfReader.getPdfObject(acroForm.get(PdfName.DR));
-					if( dr!= null && dr.isDictionary() ) {
+			PdfObject catalog_obj= reader.getCatalog();
+			if( catalog_obj!= null && catalog_obj.isDictionary() ) {
+				PdfDictionary catalog= (PdfDictionary)catalog_obj;
+				PdfObject acroForm_obj= PdfReader.getPdfObject(catalog.get(PdfName.ACROFORM));
+				if( acroForm_obj!= null && acroForm_obj.isDictionary() ) {
+					PdfDictionary acroForm= (PdfDictionary)acroForm_obj;
+					PdfObject dr_obj= PdfReader.getPdfObject(acroForm.get(PdfName.DR));
+					if( dr_obj!= null && dr_obj.isDictionary() ) {
+						PdfDictionary dr= (PdfDictionary)dr_obj;
 						PdfDictionary acroForm_target= this.getAcroForm();
 						PdfDictionary dr_target= (PdfDictionary)PdfReader.getPdfObject(acroForm_target.get(PdfName.DR));
 						if( dr_target== null ) {
