@@ -56,6 +56,7 @@ package pdftk.com.lowagie.text.pdf;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.Iterator;
@@ -706,10 +707,14 @@ public class PdfStamperImp extends PdfWriter {
         if (append)
             throw new IllegalArgumentException("Field flattening is not supported in append mode.");
         getAcroFields();
-        HashMap fields = acroFields.getFields();
+        HashMap<String,AcroFields.Item> fields = acroFields.getFields();
+        String[] fieldnames = fields.keySet().toArray(new String[0]);
+        // Flatten fields in hierarchical order (parent first)
+        // See https://gitlab.com/pdftk-java/pdftk/-/issues/71
+        Arrays.sort(fieldnames);
         if (fieldsAdded && partialFlattening.isEmpty()) {
-            for (Iterator i = fields.keySet().iterator(); i.hasNext();) {
-                partialFlattening.add(i.next());
+            for (String name : fieldnames) {
+                partialFlattening.add(name);
             }
         }
         PdfDictionary acroForm = (PdfDictionary)PdfReader.getPdfObject(reader.getCatalog().get(PdfName.ACROFORM));
@@ -719,11 +724,10 @@ public class PdfStamperImp extends PdfWriter {
             if (array != null)
                 acroFds = array.getArrayList();
         }
-        for (Iterator i = fields.keySet().iterator(); i.hasNext();) {
-            String name = (String)i.next();
+        for (String name : fieldnames) {
             if (!partialFlattening.isEmpty() && !partialFlattening.contains(name))
                 continue;
-            AcroFields.Item item = (AcroFields.Item)fields.get(name);
+            AcroFields.Item item = fields.get(name);
             for (int k = 0; k < item.merged.size(); ++k) {
                 PdfDictionary merged = (PdfDictionary)item.merged.get(k);
                 PdfNumber ff = (PdfNumber)PdfReader.getPdfObject(merged.get(PdfName.F));
@@ -742,17 +746,15 @@ public class PdfStamperImp extends PdfWriter {
                         ((PdfDictionary)objReal).put(PdfName.SUBTYPE, PdfName.FORM);
                         app = new PdfAppearance((PdfIndirectReference)obj);
                     }
-                    else {
-                        if (objReal.isDictionary()) {
-                            PdfName as = (PdfName)PdfReader.getPdfObject(merged.get(PdfName.AS));
-                            if (as != null) {
-                                PdfIndirectReference iref = (PdfIndirectReference)((PdfDictionary)objReal).get(as);
-                                if (iref != null) {
-                                    app = new PdfAppearance(iref);
-                                    if (iref.isIndirect()) {
-                                        objReal = PdfReader.getPdfObject(iref);
-                                        ((PdfDictionary)objReal).put(PdfName.SUBTYPE, PdfName.FORM);
-                                    }
+                    else if (objReal != null && objReal.isDictionary()) {
+                        PdfName as = (PdfName)PdfReader.getPdfObject(merged.get(PdfName.AS));
+                        if (as != null) {
+                            PdfIndirectReference iref = (PdfIndirectReference)((PdfDictionary)objReal).get(as);
+                            if (iref != null) {
+                                app = new PdfAppearance(iref);
+                                if (iref.isIndirect()) {
+                                    objReal = PdfReader.getPdfObject(iref);
+                                    ((PdfDictionary)objReal).put(PdfName.SUBTYPE, PdfName.FORM);
                                 }
                             }
                         }
